<?php
/**
 * Plugin Name: Shop as Client for WooCommerce
 * Plugin URI: https://www.webdados.pt/produtos-e-servicos/internet/desenvolvimento-wordpress/shop-as-client-for-woocommerce/
 * Description: Allows a WooCommerce Store Administrator or Shop Manager to use the frontend and assign a new order to a registered or new customer. Useful for phone or email orders.
 * Version: 0.1
 * Author: Webdados
 * Author URI: https://www.webdados.pt
 * Developer: Marco Almeida
 * Text Domain: shop-as-client
 * Domain Path: /languages
 * Requires at least: 4.7
 * WC requires at least: 2.1.0
 * WC tested up to: 3.5.0
**/

/* WooCommerce CRUD ready */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Check if WooCommerce is active
 **/
// Get active network plugins - "Stolen" from Novalnet Payment Gateway
function sac_active_nw_plugins() {
	if ( !is_multisite() )
		return false;
	$sac_activePlugins = ( get_site_option( 'active_sitewide_plugins' ) ) ? array_keys( get_site_option( 'active_sitewide_plugins' ) ) : array();
	return $sac_activePlugins;
}
if ( in_array( 'woocommerce/woocommerce.php', (array) get_option( 'active_plugins ') ) || in_array( 'woocommerce/woocommerce.php', (array) sac_active_nw_plugins() ) ) {

	//Languages
	add_action( 'plugins_loaded', 'shop_as_client_init' );
	function shop_as_client_init() {
		load_plugin_textdomain( 'shop-as-client' );
	}

	//Our field
	add_filter( 'woocommerce_billing_fields' , 'shop_as_client_init_woocommerce_billing_fields' );
	function shop_as_client_init_woocommerce_billing_fields( $fields ) {
		if ( current_user_can( 'manage_woocommerce' ) && is_checkout() ) {
			//Shop as client?
			$fields['billing_shop_as_client'] = array(
				'label'		=> __( 'Shop as client', 'shop-as-client' ),
				'required'	=> true,
				'class'		=> array( 'form-row-wide' ),
				'clear'		=> true,
				'priority'	=> 990,
				'type'		=> 'select',
				'options'	=> array(
					'yes'	=> __( 'Yes', 'shop-as-client' ),
					'no'	=> __( 'No', 'shop-as-client' ),
				),
				'default'	=> apply_filters( 'shop_as_client_default_shop_as_client', 'yes' ),
			);
			//Create user if it doesn't exist?
			$fields['billing_shop_as_client_create_user'] = array(
				'label'		=> __( 'Create user (if not found by email)?', 'shop-as-client' ),
				'required'	=> true,
				'class'		=> array( 'form-row-wide' ),
				'clear'		=> true,
				'priority'	=> 991,
				'type'		=> 'select',
				'options'	=> array(
					'yes'	=> __( 'Yes', 'shop-as-client' ),
					'no'	=> __( 'No (leave as guest)', 'shop-as-client' ),
				),
				'default'	=> apply_filters( 'shop_as_client_default_create_user', 'no' ),
			);
		}
		return $fields;
	}

	//Get order "shop as client"
	function shop_as_client_get_order_status( $order ) {
		return 'yes' == version_compare( WC_VERSION, '3.0', '>=' ) ? $order->get_meta( '_billing_shop_as_client' ) : $order->billing_shop_as_client;
	}

	//Return yes to woocommerce_registration_generate_password
	function shop_as_client_woocommerce_registration_generate_password( $value ) {
		return 'yes';
	}

	//Adjust user - Inspiration: https://gist.github.com/twoelevenjay/80294a635969a54e4693
	add_filter( 'woocommerce_checkout_customer_id', 'shop_as_client_woocommerce_checkout_customer_id' );
	function shop_as_client_woocommerce_checkout_customer_id( $user_id ) {
		if ( current_user_can( 'manage_woocommerce' ) ) {
			if ( isset( $_POST['billing_shop_as_client'] ) && 'yes' == $_POST['billing_shop_as_client'] ) {
				 // Check if an exisiting user already uses this email address.
				$user_email = sanitize_email( $_POST['billing_email'] );
				if ( $user = get_user_by( 'email', $user_email ) ) {
					//User found
					$user_id = $user->ID;
				} else {
					//Create user or guest?
					if ( isset( $_POST['billing_shop_as_client_create_user'] ) && 'yes' == $_POST['billing_shop_as_client_create_user'] ) {
						//Username
						if ( 'yes' === get_option( 'woocommerce_registration_generate_username' ) ) {
							$username = '';
						} else {
							$username = $user_email;
						}
						//Force password generation by WooCommerce (and sending via email), even if the option is not set
						if ( apply_filters( 'shop_as_client_email_password', true ) ) {
							add_filter( 'option_woocommerce_registration_generate_password', 'shop_as_client_woocommerce_registration_generate_password' );
							$password = '';
						} else {
							$password = wp_generate_password();
						}
						$user_id = wc_create_new_customer( $user_email, $username, $password );
						if ( apply_filters( 'shop_as_client_email_password', true ) ) remove_filter( 'option_woocommerce_registration_generate_password', 'shop_as_client_woocommerce_registration_generate_password' );
						if ( !is_wp_error( $user_id ) ) {
							$user_first_name = sanitize_text_field( $_POST['billing_first_name'] );
							$user_last_name = sanitize_text_field( $_POST['billing_last_name'] );
							wp_update_user(
								array( 'ID' => $user_id,
									'first_name' => $user_first_name,
									'last_name' => $user_last_name,
									'display_name' => trim( $user_first_name.' '.$user_last_name ),
									//'role' => 'customer',
								)
							);
						} else {
							$user_id = 0;
						}
					} else {
						$user_id = 0;
					}
				}
			}
		}
		return $user_id;
	}

	//Information on the order edit screen
	add_action( 'woocommerce_admin_order_data_after_order_details', 'shop_as_client_woocommerce_admin_order_data_after_order_details' );
	function shop_as_client_woocommerce_admin_order_data_after_order_details( $order ) {
		if ( shop_as_client_get_order_status( $order ) ) {
			?>
			<p class="form-field form-field-wide">
				<label><?php _e( 'Shop as client', 'shop-as-client' ) ?>:</label>
				<?php _e( 'Yes', 'shop-as-client' ) ?>
			</p>
			<?php
		}
	}

	/* If you're reading this you must know what you're doing ;-) Greetings from sunny Portugal! */
	
}